﻿using System;
using System.IO;
using System.Collections.Generic;
using Bentley.GeometryNET;
using Bentley.DgnPlatformNET;
using Bentley.DgnPlatformNET.Elements;
using Bentley.MstnPlatformNET;

using BCOM = Bentley.Interop.MicroStationDGN;

namespace csAddins
{
    class CreateDimensionCallbacks : DimensionCreateData
    {
        private DimensionStyle m_dimStyle;
        private DgnTextStyle m_textStyle;
        private Symbology m_symbology;
        private LevelId m_levelId;
        private DirectionFormatter m_directionFormatter;

        public CreateDimensionCallbacks(DimensionStyle dimStyle, DgnTextStyle textStyle, Symbology symb, LevelId levelId, DirectionFormatter formatter)
        {
            m_dimStyle = dimStyle;
            m_textStyle = textStyle;
            m_symbology = symb;
            m_levelId = levelId;
            m_directionFormatter = formatter;
        }
        public override DimensionStyle GetDimensionStyle()
        {
            return m_dimStyle;
        }
        public override DgnTextStyle GetTextStyle()
        {
            return m_textStyle;
        }
        public override Symbology GetSymbology()
        {
            return m_symbology;
        }
        public override LevelId GetLevelId()
        {
            return m_levelId;
        }
        public override int GetViewNumber()
        {
            return 0;
        }
        public override DMatrix3d GetDimensionRotation()
        {
            return DMatrix3d.Identity;
        }
        public override DMatrix3d GetViewRotation()
        {
            return DMatrix3d.Identity;
        }
        public override DirectionFormatter GetDirectionFormatter()
        {
            return m_directionFormatter;
        }
    }

    class CreateElement
    {
        public static void LineAndLineString(string unparsed)
        {
            DgnModel oModel = Session.Instance.GetActiveDgnModel();
            double uorPerMast = oModel.GetModelInfo().UorPerMaster;

            DSegment3d oSegment = new DSegment3d(0, 0, 0, 10 * uorPerMast, 0, 0);
            LineElement oLine = new LineElement(oModel, null, oSegment);
            ElementPropertiesSetter propSetter = new ElementPropertiesSetter();
            propSetter.SetColor(0);
            propSetter.SetWeight(2);
            propSetter.Apply(oLine);
            oLine.AddToModel();

            DPoint3d[] pntArray = new DPoint3d[5];
            pntArray[0] = DPoint3d.FromXY(0, 0);
            pntArray[1] = DPoint3d.FromXY(uorPerMast, 2 * uorPerMast);
            pntArray[2] = DPoint3d.FromXY(3 * uorPerMast, -2 * uorPerMast);
            pntArray[3] = DPoint3d.FromXY(5 * uorPerMast, 2 * uorPerMast);
            pntArray[4] = DPoint3d.FromXY(6 * uorPerMast, 0);
            LineStringElement oLineString = new LineStringElement(oModel, null, pntArray);
            propSetter.SetColor(1);
            propSetter.Apply(oLineString);
            oLineString.AddToModel();
        }
        public static void ShapeAndComplexShape(string unparsed)
        {
            DgnModel oModel = Session.Instance.GetActiveDgnModel();
            double uorPerMast = oModel.GetModelInfo().UorPerMaster;

            DPoint3d[] pntArray = new DPoint3d[6];
            pntArray[0] = DPoint3d.FromXY(0, -6 * uorPerMast);
            pntArray[1] = DPoint3d.FromXY(0, -2 * uorPerMast);
            pntArray[2] = DPoint3d.FromXY(2 * uorPerMast, -2 * uorPerMast);
            pntArray[3] = DPoint3d.FromXY(2 * uorPerMast, -4 * uorPerMast);
            pntArray[4] = DPoint3d.FromXY(4 * uorPerMast, -4 * uorPerMast);
            pntArray[5] = DPoint3d.FromXY(4 * uorPerMast, -6 * uorPerMast);
            ShapeElement oShape = new ShapeElement(oModel, null, pntArray);
            ElementPropertiesSetter propSetter = new ElementPropertiesSetter();
            propSetter.SetColor(0);
            propSetter.SetWeight(2);
            propSetter.Apply(oShape);
            oShape.AddToModel();

            ComplexShapeElement oComplexShape = new ComplexShapeElement (oModel, null);
            for (int i = 0; i < 6; i++)
                pntArray[i].X += 5 * uorPerMast;
            LineStringElement oLineString = new LineStringElement(oModel, null, pntArray);
            oComplexShape.AddComponentElement (oLineString);

            DPoint3d center = DPoint3d.Interpolate (pntArray[0], 0.5, pntArray[5]);
            double radius = pntArray[0].Distance(pntArray[5]) / 2.0;
            ArcElement oArc = new ArcElement(oModel, null, center, radius, radius, 0.0, 0.0, -Angle.PI.Radians);
            oComplexShape.AddComponentElement (oArc);
            oComplexShape.AddComponentComplete();
            propSetter.SetColor(1);
            propSetter.Apply(oComplexShape);
            oComplexShape.AddToModel();
        }
        public static void TextAndTextNode(string unparsed)
        {
            DgnModel oModel = Session.Instance.GetActiveDgnModel();
            double uorPerMast = oModel.GetModelInfo().UorPerMaster;

            TextBlockProperties txtBlkProp = new TextBlockProperties(oModel);
            ParagraphProperties paraProp = new ParagraphProperties(oModel);
            DgnFont      myFont = DgnFontManager.FindSystemFont("Arial", DgnFontFilterFlags.TrueType);
            RunProperties runProp = new RunProperties(myFont, DPoint2d.FromXY(0.7 * uorPerMast, 0.6 * uorPerMast), oModel);
            TextBlock txtBlk = new TextBlock(txtBlkProp, paraProp, runProp, oModel);
            txtBlk.AppendText("Text String");
            txtBlk.SetUserOrigin(DPoint3d.FromXY(0, -7.5 * uorPerMast));
            TextHandlerBase oText = TextHandlerBase.CreateElement(null, txtBlk);
            ElementPropertiesSetter propSetter = new ElementPropertiesSetter();
            propSetter.SetColor(0);
            propSetter.Apply(oText);
            oText.AddToModel();

            TextBlock txtBlk2 = new TextBlock(txtBlkProp, paraProp, runProp, oModel);
            txtBlk2.AppendText("Text Node Line 1");
            txtBlk2.AppendLineBreak();
            txtBlk2.AppendText("Text Node Line 2");
            txtBlk2.SetUserOrigin(DPoint3d.FromXY(2 * uorPerMast, -9 * uorPerMast));
            TextHandlerBase oTextNode = TextHandlerBase.CreateElement(null, txtBlk2);
            propSetter.SetColor(1);
            propSetter.Apply(oTextNode);
            oTextNode.AddToModel();
        }
        public static void CellAndSharedCell(string unparsed)
        {
            BCOM.Application app = Bentley.MstnPlatformNET.InteropServices.Utilities.ComApp;
            app.AttachCellLibrary("sample2.cel");
            BCOM.Point3d origin = app.Point3dFromXY(1, -13);
            double xScale = 0.1 * app.ActiveModelReference.UORsPerMasterUnit / 1000.0;
            BCOM.Point3d scale = app.Point3dFromXYZ(xScale, xScale, xScale);
            BCOM.Matrix3d rMatrix = app.Matrix3dIdentity();
            BCOM.CellElement oCell = app.CreateCellElement2("DECID", ref origin, ref scale, true, ref rMatrix);
            oCell.Color = 0;
            app.ActiveModelReference.AddElement(oCell);

            BCOM.SharedCellElement oSC;
            xScale = 0.02 * app.ActiveModelReference.UORsPerMasterUnit / 1000.0;
            scale = app.Point3dFromXYZ(xScale, xScale, xScale);
            for (int x = 4; x <= 8; x += 2)
            {
                origin = app.Point3dFromXY(x, -14);
                oSC = app.CreateSharedCellElement2("NORTH", ref origin, ref scale, true, ref rMatrix);
                oSC.OverridesComponentColor = true;     oSC.Color = 1;
                app.ActiveModelReference.AddElement(oSC);
            }
        }
        public static void LinearAndAngularDimension(string unparsed)
        {
            DgnFile oFile = Session.Instance.GetActiveDgnFile();
            DgnModel oModel = Session.Instance.GetActiveDgnModel();
            double uorPerMast = oModel.GetModelInfo().UorPerMaster;
            
            DimensionStyle dimStyle = new DimensionStyle("DimStyle", oFile);
            dimStyle.SetBooleanProp(true, DimStyleProp.Placement_UseStyleAnnotationScale_BOOLINT);
            dimStyle.SetDoubleProp(1, DimStyleProp.Placement_AnnotationScale_DOUBLE);
            dimStyle.SetBooleanProp(true, DimStyleProp.Text_OverrideHeight_BOOLINT);
            dimStyle.SetDoubleProp(0.5 * uorPerMast, DimStyleProp.Text_Height_DISTANCE);
            dimStyle.SetBooleanProp(true, DimStyleProp.Text_OverrideWidth_BOOLINT);
            dimStyle.SetDoubleProp(0.4 * uorPerMast, DimStyleProp.Text_Width_DISTANCE);
            dimStyle.SetBooleanProp(true, DimStyleProp.General_UseMinLeader_BOOLINT);
            dimStyle.SetDoubleProp(0.01, DimStyleProp.Terminator_MinLeader_DOUBLE);
            dimStyle.SetBooleanProp(true, DimStyleProp.Value_AngleMeasure_BOOLINT);
            dimStyle.SetAccuracyProp ((byte)AnglePrecision.Use1Place, DimStyleProp.Value_AnglePrecision_INTEGER);

            DgnTextStyle textStyle = new DgnTextStyle("TestStyle", oFile);
            LevelId lvlId = Settings.GetLevelIdFromName("Default");
            CreateDimensionCallbacks callbacks = new CreateDimensionCallbacks(dimStyle, textStyle, new Symbology(), lvlId, null);
            DimensionElement oDim = new DimensionElement(oModel, callbacks, DimensionType.SizeArrow);
            if (oDim.IsValid)
            {
                DPoint3d pt = new DPoint3d(0, -17 * uorPerMast, 0);
                oDim.InsertPoint(pt, null, dimStyle, -1);
                pt.X += 3*uorPerMast;
                oDim.InsertPoint(pt, null, dimStyle, -1);
                oDim.SetHeight(uorPerMast);
                oDim.AddToModel();
            }
            oDim = new DimensionElement(oModel, callbacks, DimensionType.AngleSize);
            if (oDim.IsValid)
            {
                DPoint3d pt = new DPoint3d(7*uorPerMast, -13 * uorPerMast, 0);
                oDim.InsertPoint(pt, null, dimStyle, -1);
                pt = DPoint3d.FromXY (5*uorPerMast, -15*uorPerMast);
                oDim.InsertPoint(pt, null, dimStyle, -1);
                pt = DPoint3d.FromXY (9*uorPerMast, -15*uorPerMast);
                oDim.InsertPoint(pt, null, dimStyle, -1);
                oDim.SetHeight(uorPerMast);
                oDim.AddToModel();
            }
        }
        public static void CurveAndBsplineCurve(string unparsed)
        {
            DgnModel oModel = Session.Instance.GetActiveDgnModel();
            double uorPerMast = oModel.GetModelInfo().UorPerMaster;

            DPoint3d[] pntArray = new DPoint3d[5];
            pntArray[0] = DPoint3d.FromXY(0, -19 * uorPerMast);
            pntArray[1] = DPoint3d.FromXY(uorPerMast, -17 * uorPerMast);
            pntArray[2] = DPoint3d.FromXY(2 * uorPerMast, -19 * uorPerMast);
            pntArray[3] = DPoint3d.FromXY(3 * uorPerMast, -17 * uorPerMast);
            pntArray[4] = DPoint3d.FromXY(4 * uorPerMast, -19 * uorPerMast);
            //CurveElement oCurve = new CurveElement(oModel, null, pntArray);   // Crash MicroStation
            ElementPropertiesSetter propSetter = new ElementPropertiesSetter();
            propSetter.SetColor(0);
            propSetter.SetWeight(2);
            //propSetter.Apply(oCurve);
            //oCurve.AddToModel();

            for (int i = 0; i < 5; i++)
                pntArray[i].X += 5 * uorPerMast;
            MSBsplineCurve msBSCurve = MSBsplineCurve.CreateFromPoles(pntArray, null, null, 3, false, false);
            if (BSplineStatus.Success == BSplineCurveElement.IsValidCurve(msBSCurve))
            {
                BSplineCurveElement oBSCurve = new BSplineCurveElement(oModel, null, msBSCurve);
                propSetter.SetColor(1);
                propSetter.Apply(oBSCurve);
                oBSCurve.AddToModel();
            }
        }
        public static void ConeAndBsplineSurface(string unparsed)
        {
            BCOM.Application app = Bentley.MstnPlatformNET.InteropServices.Utilities.ComApp;
            DgnModel oModel = Session.Instance.GetActiveDgnModel();
            double uorPerMast = oModel.GetModelInfo().UorPerMaster;

            DPoint3d basePt = DPoint3d.FromXYZ(2*uorPerMast, -23*uorPerMast, 0);
            DPoint3d topPt = DPoint3d.FromXYZ(2*uorPerMast, -20*uorPerMast, 0);
            BCOM.Matrix3d BCOMMatrix = app.Matrix3dFromAxisAndRotationAngle(0, app.Pi() / 6);
            DMatrix3d rMatrix = new DMatrix3d(BCOMMatrix.RowX.X, BCOMMatrix.RowX.Y, BCOMMatrix.RowX.Z,
                                              BCOMMatrix.RowY.X, BCOMMatrix.RowY.Y, BCOMMatrix.RowY.Z,
                                              BCOMMatrix.RowZ.X, BCOMMatrix.RowZ.Y, BCOMMatrix.RowZ.Z);
            ConeElement oCone = new ConeElement(oModel, null, uorPerMast, 2 * uorPerMast, topPt, basePt, rMatrix, true);
            oCone.AddToModel();

            List<DPoint3d> pts = new List<DPoint3d>();
            pts.Add(DPoint3d.FromXYZ(5.9*uorPerMast, -21*uorPerMast, -0.5*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(6.9*uorPerMast, -20*uorPerMast, uorPerMast));
            pts.Add(DPoint3d.FromXYZ(7.9*uorPerMast, -20.3*uorPerMast, 1.3*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(8.9*uorPerMast, -20.8*uorPerMast, 0.3*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(6.4*uorPerMast, -22*uorPerMast, 0));
            pts.Add(DPoint3d.FromXYZ(7.1*uorPerMast, -21.2*uorPerMast, 0.7*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(7.7*uorPerMast, -21*uorPerMast, uorPerMast));
            pts.Add(DPoint3d.FromXYZ(8.4*uorPerMast, -21.7*uorPerMast, -0.2*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(5.9*uorPerMast, -23*uorPerMast, 0));
            pts.Add(DPoint3d.FromXYZ(7.2*uorPerMast, -23.1*uorPerMast, 1.2*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(7.8*uorPerMast, -23.3*uorPerMast, 0.8*uorPerMast));
            pts.Add(DPoint3d.FromXYZ(8.7*uorPerMast, -22.8*uorPerMast, 0.2*uorPerMast));
            int UPoleCnt = 4, VPoleCnt = 3, order = 3;
            double[] UKnots = new double[] { 0, 0, 0, 0.5, 1, 1, 1};
            double[] VKnots = new double[] { 0, 0, 0, 1, 1, 1};
            MSBsplineSurface msSurface = MSBsplineSurface.CreateFromPoles(pts, null, UKnots, order, UPoleCnt, false,
                                                                                     VKnots, order, VPoleCnt, false, false);
            if (BSplineSurfaceElement.IsValidSurface(msSurface) == BSplineStatus.Success)
            {
                BSplineSurfaceElement oSurface = new BSplineSurfaceElement(oModel, null, msSurface);
                oSurface.AddToModel();
            }
        }

        public static void Mesh(string unparsed)
        {
            DgnModel oModel = Session.Instance.GetActiveDgnModel();

            char[] delimiterChars = { ' ' };
            string myLine;
            List<DPoint3d> meshPnts = new List<DPoint3d>();
            StreamReader sr = new StreamReader(@"d:\data-13.asc");
            while (null != (myLine = sr.ReadLine()))
            {
                string[] sArray = myLine.Split(delimiterChars);
                double x = double.Parse(sArray[0]);
                double y = double.Parse(sArray[1]);
                double z = double.Parse(sArray[2]);
                meshPnts.Add(DPoint3d.FromXYZ(x, y, z));
            }
            sr.Close();
            PolyfaceConstruction polyCons = new PolyfaceConstruction();
            polyCons.AddTriangulation(meshPnts);  //or other Add methods
            MeshHeaderElement oMesh = new MeshHeaderElement(oModel, null, polyCons.GetClientMesh());
            oMesh.AddToModel();
        }
    }
}
